@@ -42,6 +42,7 @@
 #include "gstomxaacdec.h"
 #include "gstomxmp3dec.h"
 #include "gstomxaacenc.h"
+#include "gstomxamrdec.h"
 #include "gstomxanalogaudiosink.h"
 #include "gstomxhdmiaudiosink.h"
 
@@ -829,6 +830,13 @@ gst_omx_component_set_state (GstOMXComponent * comp, OMX_STATETYPE state)
 done:
 
   gst_omx_component_handle_messages (comp);
+
+  if (err != OMX_ErrorNone && comp->last_error == OMX_ErrorNone) {
+    GST_ERROR_OBJECT (comp->parent,
+        "Last operation returned an error. Setting last_error manually.");
+    comp->last_error = err;
+  }
+
   g_mutex_unlock (&comp->lock);
 
   if (err != OMX_ErrorNone) {
@@ -1196,7 +1204,7 @@ OMX_ERRORTYPE
 gst_omx_port_update_port_definition (GstOMXPort * port,
     OMX_PARAM_PORTDEFINITIONTYPE * port_def)
 {
-  OMX_ERRORTYPE err = OMX_ErrorNone;
+  OMX_ERRORTYPE err_get, err_set = OMX_ErrorNone;
   GstOMXComponent *comp;
 
   g_return_val_if_fail (port != NULL, FALSE);
@@ -1204,16 +1212,19 @@ gst_omx_port_update_port_definition (GstOMXPort * port,
   comp = port->comp;
 
   if (port_def)
-    err =
+    err_set =
         gst_omx_component_set_parameter (comp, OMX_IndexParamPortDefinition,
         port_def);
-  gst_omx_component_get_parameter (comp, OMX_IndexParamPortDefinition,
+  err_get = gst_omx_component_get_parameter (comp, OMX_IndexParamPortDefinition,
       &port->port_def);
 
   GST_DEBUG_OBJECT (comp->parent, "Updated %s port %u definition: %s (0x%08x)",
-      comp->name, port->index, gst_omx_error_to_string (err), err);
+      comp->name, port->index, gst_omx_error_to_string (err_set), err_set);
 
-  return err;
+  if (err_set != OMX_ErrorNone)
+    return err_set;
+  else
+    return err_get;
 }
 
 /* NOTE: Uses comp->lock and comp->messages_lock */
@@ -1224,6 +1235,7 @@ gst_omx_port_acquire_buffer (GstOMXPort * port, GstOMXBuffer ** buf)
   GstOMXComponent *comp;
   OMX_ERRORTYPE err;
   GstOMXBuffer *_buf = NULL;
+  gint64 timeout = GST_CLOCK_TIME_NONE;
 
   g_return_val_if_fail (port != NULL, GST_OMX_ACQUIRE_BUFFER_ERROR);
   g_return_val_if_fail (!port->tunneled, GST_OMX_ACQUIRE_BUFFER_ERROR);
@@ -1240,6 +1252,11 @@ gst_omx_port_acquire_buffer (GstOMXPort * port, GstOMXBuffer ** buf)
 retry:
   gst_omx_component_handle_messages (comp);
 
+  /* If we are in the case where we waited for a buffer after EOS,
+   * make sure we don't do that again */
+  if (timeout != -1)
+    timeout = -2;
+
   /* Check if the component is in an error state */
   if ((err = comp->last_error) != OMX_ErrorNone) {
     GST_ERROR_OBJECT (comp->parent, "Component %s is in error state: %s",
@@ -1250,6 +1267,8 @@ retry:
 
   /* Check if the port is flushing */
   if (port->flushing) {
+    GST_DEBUG_OBJECT (comp->parent, "Component %s port %d is flushing",
+        comp->name, port->index);
     ret = GST_OMX_ACQUIRE_BUFFER_FLUSHING;
     goto done;
   }
@@ -1275,6 +1294,8 @@ retry:
     /* Only if this port needs to be reconfigured too notify
      * the caller about it */
     if (port->settings_cookie != port->configured_settings_cookie) {
+      GST_DEBUG_OBJECT (comp->parent,
+          "Component %s port %d needs reconfiguring", comp->name, port->index);
       ret = GST_OMX_ACQUIRE_BUFFER_RECONFIGURE;
       goto done;
     }
@@ -1297,6 +1318,8 @@ retry:
       goto done;
     }
 
+    GST_DEBUG_OBJECT (comp->parent, "Component %s port %d needs reconfiguring",
+        comp->name, port->index);
     ret = GST_OMX_ACQUIRE_BUFFER_RECONFIGURE;
     goto done;
   }
@@ -1311,11 +1334,21 @@ retry:
       goto done;
     }
 
-    ret = GST_OMX_ACQUIRE_BUFFER_EOS;
-    goto done;
+    if (comp->hacks & GST_OMX_HACK_SIGNALS_PREMATURE_EOS && timeout != -2) {
+      timeout = 33 * GST_MSECOND;
+
+      GST_DEBUG_OBJECT (comp->parent, "%s output port %u is EOS but waiting "
+          "in case it spits out more buffers", comp->name, port->index);
+    } else {
+      GST_DEBUG_OBJECT (comp->parent, "Component %s port %d signalled EOS",
+          comp->name, port->index);
+      ret = GST_OMX_ACQUIRE_BUFFER_EOS;
+      port->eos = FALSE;
+      goto done;
+    }
   }
 
-  /* 
+  /*
    * At this point we have no error or flushing/eos port
    * and a properly configured port.
    *
@@ -1325,25 +1358,20 @@ retry:
    * arrives, an error happens, the port is flushing
    * or the port needs to be reconfigured.
    */
-  gst_omx_component_handle_messages (comp);
   if (g_queue_is_empty (&port->pending_buffers)) {
     GST_DEBUG_OBJECT (comp->parent, "Queue of %s port %u is empty",
         comp->name, port->index);
-    gst_omx_component_wait_message (comp, GST_CLOCK_TIME_NONE);
-    gst_omx_component_handle_messages (comp);
+    gst_omx_component_wait_message (comp,
+        timeout == -2 ? GST_CLOCK_TIME_NONE : timeout);
 
     /* And now check everything again and maybe get a buffer */
     goto retry;
-  } else {
-    GST_DEBUG_OBJECT (comp->parent, "%s port %u has pending buffers",
-        comp->name, port->index);
-    _buf = g_queue_pop_head (&port->pending_buffers);
-    ret = GST_OMX_ACQUIRE_BUFFER_OK;
-    goto done;
   }
 
-  g_assert_not_reached ();
-  goto retry;
+  GST_DEBUG_OBJECT (comp->parent, "%s port %u has pending buffers",
+      comp->name, port->index);
+  _buf = g_queue_pop_head (&port->pending_buffers);
+  ret = GST_OMX_ACQUIRE_BUFFER_OK;
 
 done:
   g_mutex_unlock (&comp->lock);
@@ -1368,7 +1396,7 @@ gst_omx_port_release_buffer (GstOMXPort * port, GstOMXBuffer * buf)
   OMX_ERRORTYPE err = OMX_ErrorNone;
 
   g_return_val_if_fail (port != NULL, OMX_ErrorUndefined);
-  g_return_val_if_fail (!port->tunneled, GST_OMX_ACQUIRE_BUFFER_ERROR);
+  g_return_val_if_fail (!port->tunneled, OMX_ErrorUndefined);
   g_return_val_if_fail (buf != NULL, OMX_ErrorUndefined);
   g_return_val_if_fail (buf->port == port, OMX_ErrorUndefined);
 
@@ -2247,7 +2275,8 @@ static const GGetTypeFunction types[] = {
   gst_omx_wmv_dec_get_type, gst_omx_mpeg4_video_enc_get_type,
   gst_omx_h264_enc_get_type, gst_omx_h263_enc_get_type,
   gst_omx_aac_enc_get_type, gst_omx_mjpeg_dec_get_type,
-  gst_omx_aac_dec_get_type, gst_omx_mp3_dec_get_type
+  gst_omx_aac_dec_get_type, gst_omx_mp3_dec_get_type,
+  gst_omx_amr_dec_get_type
 #ifdef HAVE_VP8
       , gst_omx_vp8_dec_get_type
 #endif
@@ -2459,6 +2488,8 @@ gst_omx_parse_hacks (gchar ** hacks)
       hacks_flags |= GST_OMX_HACK_NO_COMPONENT_ROLE;
     else if (g_str_equal (*hacks, "no-disable-outport"))
       hacks_flags |= GST_OMX_HACK_NO_DISABLE_OUTPORT;
+    else if (g_str_equal (*hacks, "signals-premature-eos"))
+      hacks_flags |= GST_OMX_HACK_SIGNALS_PREMATURE_EOS;
     else
       GST_WARNING ("Unknown hack: %s", *hacks);
     hacks++;
@@ -2619,6 +2650,7 @@ _class_init (gpointer g_class, gpointer data)
 #endif
 
     class_data->hacks = gst_omx_parse_hacks (hacks);
+    g_strfreev (hacks);
   }
 }
 
@@ -2636,6 +2668,7 @@ plugin_init (GstPlugin * plugin)
   gsize n_elements;
   static const gchar *config_name[] = { "gstomx.conf", NULL };
   static const gchar *env_config_name[] = { "GST_OMX_CONFIG_DIR", NULL };
+  static const gchar *gst_omx_config_dir = GST_OMX_CONFIG_DIR;
 
   GST_DEBUG_CATEGORY_INIT (gstomx_debug, "omx", 0, "gst-omx");
   GST_DEBUG_CATEGORY_INIT (gst_omx_video_debug_category, "omxvideo", 0,
@@ -2647,7 +2680,7 @@ plugin_init (GstPlugin * plugin)
   user_config_dir = g_get_user_config_dir ();
   system_config_dirs = g_get_system_config_dirs ();
   config_dirs =
-      g_new (gchar *, g_strv_length ((gchar **) system_config_dirs) + 3);
+      g_new (gchar *, g_strv_length ((gchar **) system_config_dirs) + 4);
 
   i = 0;
   j = 0;
@@ -2656,6 +2689,7 @@ plugin_init (GstPlugin * plugin)
   config_dirs[i++] = (gchar *) user_config_dir;
   while (system_config_dirs[j])
     config_dirs[i++] = (gchar *) system_config_dirs[j++];
+  config_dirs[i++] = (gchar *) gst_omx_config_dir;
   config_dirs[i++] = NULL;
 
   gst_plugin_add_dependency (plugin, env_config_name,
